#include <QApplication>
#include "window.h"
#include <QShortcut>
#include <QStyle>

#include <QWindow>
#include <QScreen>

#include <QDebug>
#include <QSysInfo>

#include <algorithm>
#include <iostream>

// 1.0

static QString title()
{
    return QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') + QGuiApplication::platformName()
        + QLatin1Char(' ') + qApp->style()->objectName();
}

template <class Str>
static void formatScreen(const QScreen *s, Str & stream)
{
    auto g = s->geometry();
    stream << s->name() << "\", " << g.width() << 'x' << g.height() << forcesign
         << g.x() << g.y() << noforcesign << ", dpr=" << s->devicePixelRatio()
         << ", " << s->logicalDotsPerInchX() << "DPI";
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

    if (args.contains("-n")) {
        qDebug() << "AA_DisableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
    } else {
        qDebug() << "AA_EnableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    }
#if QT_VERSION >= 0x050E00
    int roundIndex = args.indexOf("-r");
    if (roundIndex >= 0 && roundIndex < args.size() - 1) {
        ++roundIndex;
        bool ok;
        int rpi = args.at(roundIndex).toInt(&ok);
        if (!ok || rpi  < 0 || rpi > int(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough)) {
            qWarning() << "Invalid RoundingPolicy" << args.at(roundIndex);
            return -1;
        }
        auto rp = static_cast<Qt::HighDpiScaleFactorRoundingPolicy>(rpi);
        qDebug() << rp;
        QGuiApplication::setHighDpiScaleFactorRoundingPolicy(rp);
    }
#endif // Qt 5.14

    Window::optParentGlOnSelf = !args.contains("-g");

    QApplication app(argc, argv);
    QString t = title() + ' ' + args.join(' ');
    {
        auto dbg = qDebug().noquote().nospace();
        dbg << t << '\n';
#if QT_VERSION >= 0x050E00
        dbg << "highDpiScaleFactorRoundingPolicy=" << QGuiApplication::highDpiScaleFactorRoundingPolicy() << '\n';
#endif // Qt 5.14
        for (auto s :  QGuiApplication::screens()) {
            formatScreen(s, dbg);
            dbg << '\n';
        }
    }

    Window window;
    // window.resize(300, 400);
    window.setWindowTitle(t);
    window.show();
    window.screenChanged();
    QObject::connect(window.windowHandle(), &QWindow::screenChanged, &window, &Window::screenChanged);

    return QCoreApplication::exec();
}
