#include <QtWidgets>
#include "window.h"
#include "myglwidget.h"

#include <QSlider>
#include <QLabel>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QFormLayout>

#include <QWindow>
#include <QScreen>

#include <QDebug>

#ifdef QT_DIAG_LIB
#  include <qwidgetdump.h>
#  include <qwindowdump.h>
#  include <nativewindowdump.h>
#endif

static QSlider *createSlider(QWidget *p)
{
    auto r = new QSlider(p);
    r->setObjectName("Slider");
    r->setOrientation(Qt::Horizontal);
    r->setRange(0, 360);
    r->setSingleStep(16);
    r->setTickPosition(QSlider::TicksAbove);
    return r;
}

bool Window::optParentGlOnSelf = true;

Window::Window(QWidget *parent) :
    QWidget(parent)
{
    setObjectName("Main");
    auto mainVLayout = new QVBoxLayout(this);
    mainVLayout->setSpacing(6);
    mainVLayout->setContentsMargins(11, 11, 11, 11);
    auto glParent =  Window::optParentGlOnSelf ? this : nullptr;
    qDebug() << __FUNCTION__ << "parenting GL on" << glParent;
    m_myGLWidget = new MyGLWidget(glParent);
    m_myGLWidget->setObjectName("GLWidget");
    mainVLayout->addWidget(m_myGLWidget);

    auto g = new QGroupBox("Controls", this);
    m_screenLabel = new QLabel(g);
    m_rotXSlider = createSlider(g);
    m_rotYSlider = createSlider(g);
    m_rotZSlider = createSlider(g);
    g->setObjectName("GroupBox");
    mainVLayout->addWidget(g);
    auto cl = new QFormLayout(g);
    cl->addRow("xRot", m_rotXSlider);
    cl->addRow("yRot", m_rotYSlider);
    cl->addRow("zRot", m_rotZSlider);
    cl->addRow(m_screenLabel);
    connect(m_myGLWidget, &MyGLWidget::xRotationChanged, m_rotXSlider, &QAbstractSlider::setValue);
    connect(m_myGLWidget, &MyGLWidget::yRotationChanged, m_rotYSlider, &QAbstractSlider::setValue);
    connect(m_myGLWidget, &MyGLWidget::zRotationChanged, m_rotZSlider, &QAbstractSlider::setValue);
    connect(m_rotXSlider, &QSlider::valueChanged, m_myGLWidget, &MyGLWidget::setXRotation);
    connect(m_rotYSlider, &QSlider::valueChanged, m_myGLWidget, &MyGLWidget::setYRotation);
    connect(m_rotZSlider, &QSlider::valueChanged, m_myGLWidget, &MyGLWidget::setZRotation);
}

Window::~Window()
{
}

void  Window::screenChanged()
{
    m_screenLabel->setText(windowHandle()->screen()->name());
}

void Window::keyPressEvent(QKeyEvent *e)
{
    auto k =  e->key();

    if (k == Qt::Key_Escape)
        close();
    else if (k == Qt::Key_D)
        dump();
    else
        QWidget::keyPressEvent(e);
}

void Window::dump()
{
#ifdef QT_DIAG_LIB
        QtDiag::dumpAllWidgets();
        QtDiag::dumpAllWindows();
        QtDiag::dumpNativeWindows(winId());
#endif
}
